<?php
// 1.3.8

function clean_ref_input ($in) {
	$search  = array("\t", "\n", "\r", '\\');
	return (str_replace ($search, '', $in));
}

function strip_www ($in) {
	$in = str_replace ('://www.', '://', $in);
	return $in;
}

function strip_indexes ($in) {
	// Strip popular indexes
	$rep = array ('index.htm', 'index.html', 'index.php', 'index.cgi', 'index.asp', 'index.jsp');
	$in = str_replace ($rep, '', $in);

	return $in;
}

function valid_ref ($in) {
	// Return good ref, else return empty
	if (strpos ('://', $in) !== FALSE) {
		return $in;
	} else {
		return '';
	}
}

function get_refs() {
	// Bailout
	if (!isset ($_SERVER['HTTP_REFERER']))
		return;

	// Config
	$log_file = 'ref.txt';
	$log_entries = 1000;
	$record_all_queries = TRUE; // Not recommended
	$host = clean_ref_input ($_SERVER['HTTP_HOST']);

	// Blacklist
	$blacklist = array (
		'google.', // Google.com, .de, .fi, etc.
		'search.', // Yahoo, mostly
		
	);

	// Ref
	// Debug: $ref = 'http://www.host.com/file.txt?query=1';
	$ref = valid_ref ($_SERVER['HTTP_REFERER']);

	// Test ref before cleaning
	if (empty ($ref))
		return 'empty';

	if ($ref === FALSE)
		return 'invalid ref';

	// Clean ref
	$ref = clean_ref_input ($ref);

	$parts = parse_url ($ref);
	$query = (!empty ($parts['query'])) ? $parts['query'] : '';
	if (strlen ($ref.$query) > 200)
		return 'long ref';

	// Check self-ref
	$stripped_host = strip_www ($host);
	$stripped_ref = strip_www ($ref);
	if (strpos ($stripped_host, $stripped_ref) !== FALSE)
		return 'self ref';

	// OK, get log file.
	// Annoying, as foreach() and array_map() don't work on scalars!
	$buffer = @file ($log_file);
	if (!empty ($buffer)) {
		if (is_array ($buffer)) {
			$buffer = array_map ('rtrim', $buffer);
		} else {
			$buffer = rtrim ($buffer);
		}

		// OK, now we scan stored refs for HTTP ref.
		// Read each buffer line, \t split, compare, update it.
		// If no match, push new ref onto $buffer.
		$total = count ($buffer);
		$match = FALSE;
		$added = 0;
		for ($i = 0; $i < $total; $i++) {
			list ($q_type, $hits, $added, $last, $stored_ref)
				= split ($buffer[$i], "\t");

			// q_type: 1 = ref, 2 = query
			if ($q_type == 1) {
				$match = $stripped_host;
			} else {
				$match = $query;
			}

			if (strpos ($match, $stored_ref) !== FALSE) {
				// Got a hit!
				// Only add ref, so we don't have more than one query per f
				$match = TRUE;
				$hits++;
				$last = time();
				$buffer[$i] = $q_type."\t".$hits."\t".$added."\t".$last."\t".$stored_ref;
				$i = $total;
			}
		}

		// New?
		// I believe array_shift() is faster than array_slice()
		if (!$match) {
			$buffer[] = "1\t1\t".time()."\t".time()."\t{$ref}";
			$added++;
			if ($record_queries && !empty ($query)) {
				$buffer[] = "2\t1\t".time()."\t".time()."\t{$query}";
				$added++;
			}
		}

		// Verify trim
		$trim = count ($buffer);
		while ($trim > $log_entries) {
			array_shift ($buffer);
			$trim--;
		}

		// Write log
		$fp = @fopen ('ref.txt', 'w');
		if ($fp) {
			reset ($buffer);
			flock ($fp, LOCK_EX);
			rewind ($fp);
			while ($trim > 0) {
				fwrite ($fp, current ($buffer)."\n");
				next ($buffer);
				$trim--;
			}
			flock ($fp, LOCK_UN);
			fclose ($fp);
		}
	}

	return;
}

?>